@@ -133,7 +133,8 @@
   "knownGeneOld3",                    "Old UCSC Genes",    NA,
   "knownGenePrevious",                "Old Known Genes",   NA,
   "ccdsGene",                         "CCDS",              NA,
-  "refGene",                          "UCSC RefSeq",       NA,
+  "refGene",                          "NCBI RefSeq",       NA,
+  "refGene",                          "RefSeq Genes",      NA,
   "xenoRefGene",                      "Other RefSeq",      NA,
   "vegaGene",                         "Vega Genes",        "Vega Protein Genes",
   "vegaPseudoGene",                   "Vega Genes",        "Vega Pseudogenes",
